@@ -6,7 +6,15 @@ use strict qw(subs vars refs);				# Make sure we can't mess up
 use warnings FATAL => 'all';				# Enable warnings to catch errors
 
 # Initialize our version
-our $VERSION = do {my@r=(q$Revision: 1.38 $=~/\d+/g);sprintf"%d."."%04d"x$#r,@r};
+our $VERSION = '0.' . sprintf( "%04d", (qw($Revision: 1049 $))[1] );
+
+# Set some constants
+BEGIN {
+	# Debug fun!
+	if ( ! defined &DEBUG ) {
+		eval "sub DEBUG () { 0 }";
+	}
+}
 
 # Import what we need
 use Carp qw( croak );
@@ -21,16 +29,6 @@ use POE::Component::Lightspeed::Hack::Events;
 # Import the message constants
 use POE::Component::Lightspeed::Constants qw( :ALL );
 
-# Set some constants
-BEGIN {
-	# Debug fun!
-	if ( ! defined &DEBUG ) {
-		eval "sub DEBUG () { 0 }";
-	} elsif ( DEBUG ) {
-		require Data::Dumper;
-	}
-}
-
 # Our global session alias
 our $SES_ALIAS = undef;
 
@@ -55,11 +53,11 @@ sub spawn {
 	if ( ! defined $SES_ALIAS or ! length( $SES_ALIAS ) ) {
 		# Debugging info...
 		if ( DEBUG ) {
-			warn 'Using default ALIAS = lightspeed';
+			warn 'Using default ALIAS = lightspeed_router';
 		}
 
 		# Set the default
-		$SES_ALIAS = 'lightspeed';
+		$SES_ALIAS = 'lightspeed_router';
 	}
 
 	# Create the graph object
@@ -88,12 +86,13 @@ sub spawn {
 			'link_up'	=>	\&Link_Up,
 
 			# Packet Action mapping
-			'ACTION_' . ACTION_HELLO . ''		=>	\&HELLO,
-			'ACTION_' . ACTION_ROUTEDEL . ''	=>	\&ROUTE_DEL,
-			'ACTION_' . ACTION_ROUTENEW . ''	=>	\&ROUTE_NEW,
-			'ACTION_' . ACTION_POST . ''		=>	\&POST,
-			'ACTION_' . ACTION_CALL . ''		=>	\&CALL,
-			'ACTION_' . ACTION_CALLREPLY . ''	=>	\&POST,
+			'ACTION_' . ACTION_HELLO . ''		=>	\&A_HELLO,
+			'ACTION_' . ACTION_ROUTEDEL . ''	=>	\&A_ROUTE_DEL,
+			'ACTION_' . ACTION_ROUTENEW . ''	=>	\&A_ROUTE_NEW,
+			'ACTION_' . ACTION_POST . ''		=>	\&A_POST,
+			'ACTION_' . ACTION_CALL . ''		=>	\&A_CALL,
+			'ACTION_' . ACTION_CALLREPLY . ''	=>	\&A_POST,
+			'ACTION_' . ACTION_INTROSPECTION . ''	=>	\&A_INTROSPECTION,
 		},
 
 		# Our own heap
@@ -286,6 +285,11 @@ sub call {
 
 			# Fire off each session!
 			foreach my $s ( @sessions ) {
+				# Check if it's a multiple-alias session
+				if ( ref( $s ) ) {
+					$s = $s->[0];
+				}
+
 				# Resolve it into a session ref
 				$s = $_[KERNEL]->_resolve_session( $s );
 
@@ -398,6 +402,11 @@ sub post_self {
 
 	# Fire off each session!
 	foreach my $s ( @sessions ) {
+		# Check if it's a multiple-alias session
+		if ( ref( $s ) ) {
+			$s = $s->[0];
+		}
+
 		# Resolve it into a session ref
 		$s = $POE::Kernel::poe_kernel->_resolve_session( $s );
 
@@ -419,6 +428,512 @@ sub post_self {
 	return 1;
 }
 
+# A link is up!
+sub Link_Up {
+	# ARG0 = wheel id, ARG1 = our kernel name, ARG2 = remote name, ARG3 = Server/Client
+	my( $ses, $id, $kernel, $remotekernel, $type ) = @_[ SENDER, ARG0 .. ARG3 ];
+	$ses = $ses->ID;
+
+	# The kernel name cannot include * or / or ,
+	if ( $kernel =~ tr|*/,|| ) {
+		die "The kernel name cannot include '*' or '/' or ','";
+	}
+
+	# Ah, now we know our name :)
+	if ( ! defined $_[HEAP]->{'MYKERNEL'} ) {
+		$_[HEAP]->{'MYKERNEL'} = $kernel;
+	} else {
+		# Sanity check...
+		if ( $_[HEAP]->{'MYKERNEL'} ne $kernel ) {
+			die 'Using different kernel names in the same process is not supported!';
+		}
+	}
+
+	# We do some sanity checks here
+	if ( $_[HEAP]->{'GRAPH'}->has_vertex( $remotekernel ) ) {
+		# A new node wants to join, but we already have the same kernel name!
+		warn "Detected same kernel name in use - $remotekernel";
+		$_[KERNEL]->call( $ses, 'killclient', $id );
+		return undef;
+	}
+
+	# Store it!
+	push( @{ $_[HEAP]->{'SID_WID'}->{ $ses } }, $id );
+	$_[HEAP]->{'WID_SID'}->{ $id } = $ses;
+	$_[HEAP]->{'WID_TYPE'}->{ $id } = $type;
+	$_[HEAP]->{'KERNEL_WID'}->{ $remotekernel } = $id;
+	$_[HEAP]->{'WID_KERNEL'}->{ $id } = $remotekernel;
+
+	# Get the edges
+	my @edges = $_[HEAP]->{'GRAPH'}->edges();
+	if ( scalar( @edges ) == 1 ) {
+		@edges = ( $edges[0]->[0], $edges[0]->[1] );
+	}
+
+	# Send our kernel info
+	$_[KERNEL]->post( $ses, 'send', $id, [
+		$remotekernel,	# MSG_TO
+		$kernel,	# MSG_FROM
+		ACTION_HELLO,	# MSG_ACTION
+		\@edges,	# MSG_DATA
+	] );
+}
+
+# A link is down!
+sub Link_Down {
+	# ARG0 = wheel id
+	my( $ses, $id ) = @_[ SENDER, ARG0 ];
+	$ses = $ses->ID;
+	my $kernel = $_[HEAP]->{'WID_KERNEL'}->{ $id };
+
+	# Remove it from that session
+	$_[HEAP]->{'SID_WID'}->{ $ses } = [ grep { $_ ne $id } @{ $_[HEAP]->{'SID_WID'}->{ $ses } } ];
+	delete $_[HEAP]->{'WID_SID'}->{ $id };
+	delete $_[HEAP]->{'SID_WID'}->{ $ses } if scalar( @{ $_[HEAP]->{'SID_WID'}->{ $ses } } ) == 0;
+
+	# Let ACTION_ROUTEDEL do some work
+	$_[KERNEL]->call( $_[SESSION], 'ACTION_' . ACTION_ROUTEDEL, [
+		$_[HEAP]->{'MYKERNEL'},			# MSG_TO
+		undef,					# MSG_FROM
+		undef,					# MSG_ACTION
+		[ $_[HEAP]->{'MYKERNEL'}, $kernel ],	# MSG_DATA
+	] );
+
+	# Remove it from the wheel mapping
+	delete $_[HEAP]->{'KERNEL_WID'}->{ $kernel };
+	delete $_[HEAP]->{'WID_KERNEL'}->{ $id };
+
+	# Are we responsible?
+	if ( $_[HEAP]->{'WID_TYPE'}->{ $id } eq 'Server' or ! $_[HEAP]->{'GRAPH'}->has_vertex( $kernel ) ) {
+		# Let all kernels know
+		SendMessage( $_[HEAP], [
+			undef,					# MSG_TO
+			undef,					# MSG_FROM
+			ACTION_ROUTEDEL,			# MSG_ACTION
+			[ $kernel, $_[HEAP]->{'MYKERNEL'} ],	# MSG_DATA
+		] );
+	}
+
+	# One last piece of deletion
+	delete $_[HEAP]->{'WID_TYPE'}->{ $id };
+
+	# All done!
+	return 1;
+}
+
+# Processes a HELLO message
+sub A_HELLO {
+	# ARG0 = data packet, ARG1 = wheel id
+	my $msg = $_[ARG0];
+
+	# Add it to our graph
+	$_[HEAP]->{'GRAPH'}->add_weighted_edge( $_[HEAP]->{'MYKERNEL'}, $msg->[ MSG_FROM ], 1 );
+
+	# Update the APSP
+	$_[HEAP]->{'APSP'} = $_[HEAP]->{'GRAPH'}->APSP_Floyd_Warshall;
+
+	# Store the new edges
+	my @edges = ();
+	if ( scalar( @{ $msg->[ MSG_DATA ] } ) > 0 ) {
+		if ( ! ref( $msg->[ MSG_DATA ]->[0] ) ) {
+			# Single edge
+			if ( ! $_[HEAP]->{'GRAPH'}->has_edge( $msg->[ MSG_DATA ]->[0], $msg->[ MSG_DATA ]->[1] ) ) {
+				push( @edges, [ $msg->[ MSG_DATA ]->[0], $msg->[ MSG_DATA ]->[1] ] );
+			}
+		} else {
+			foreach my $e ( @{ $msg->[ MSG_DATA ] } ) {
+				# Do we know about this already?
+				if ( ! $_[HEAP]->{'GRAPH'}->has_edge( $e->[0], $e->[1] ) ) {
+					# This is new to us
+					push( @edges, $e );
+				}
+			}
+		}
+	}
+
+	# This sucks.
+	my @targets = $_[HEAP]->{'GRAPH'}->vertices;
+	@targets = grep { $_ ne $msg->[ MSG_FROM ] } @targets;
+
+	# Broadcast this new info to our network
+	if ( scalar( @edges ) > 0 ) {
+		# Add them all to the graph
+		foreach my $e ( @edges ) {
+			$_[HEAP]->{'GRAPH'}->add_weighted_edge( $e->[0], $e->[1], 1 );
+		}
+
+		# Update the APSP
+		$_[HEAP]->{'APSP'} = $_[HEAP]->{'GRAPH'}->APSP_Floyd_Warshall;
+
+		# Add our link to the edges
+		push( @edges, [ $_[HEAP]->{'MYKERNEL'}, $msg->[ MSG_FROM ] ] );
+	} else {
+		# Add our link to the edges
+		@edges = ( $_[HEAP]->{'MYKERNEL'}, $msg->[ MSG_FROM ] );
+	}
+
+	# Inform the network about our new link!
+	SendMessage( $_[HEAP], [
+		\@targets,		# MSG_TO
+		undef,			# MSG_FROM
+		ACTION_ROUTENEW,	# MSG_ACTION
+		\@edges,		# MSG_DATA
+	] );
+
+	# All done!
+	return 1;
+}
+
+# Processes ROUTE_NEW messages
+sub A_ROUTE_NEW {
+	# ARG0 = data packet, ARG1 = wheel id
+	my $msg = $_[ARG0];
+
+	# Is it for us?
+	if ( FindOurself( $_[HEAP]->{'MYKERNEL'}, $msg->[ MSG_TO ] ) ) {
+		# Process each edge
+		my $edges = [];
+		if ( ! ref( $msg->[ MSG_DATA ]->[0] ) ) {
+			push( @$edges, [ $msg->[ MSG_DATA ]->[0], $msg->[ MSG_DATA ]->[1] ] );
+		} else {
+			$edges = $msg->[ MSG_DATA ];
+		}
+
+		# Do it!
+		foreach my $e ( @$edges ) {
+			# Do we know about this already?
+			if ( ! $_[HEAP]->{'GRAPH'}->has_edge( $e->[0], $e->[1] ) ) {
+				# Add it!
+				$_[HEAP]->{'GRAPH'}->add_weighted_edge( $e->[0], $e->[1], 1 );
+			} else {
+				if ( DEBUG ) {
+					warn "ROUTE_NEW -> discarding known edge: $e->[0] <=> $e->[1]";
+				}
+			}
+		}
+
+		# Update the APSP
+		$_[HEAP]->{'APSP'} = $_[HEAP]->{'GRAPH'}->APSP_Floyd_Warshall;
+	}
+
+	# Do whatever is necessary with this packet
+	SendMessage( $_[HEAP], $msg );
+
+	# All done!
+	return 1;
+}
+
+# Processes a ROUTE_DEL message
+sub A_ROUTE_DEL {
+	# ARG0 = data packet, ARG1 = wheel id
+	my $msg = $_[ARG0];
+
+	# Is it for us?
+	if ( FindOurself( $_[HEAP]->{'MYKERNEL'}, $msg->[ MSG_TO ] ) ) {
+		# Process each edge
+		my @edges = ();
+		if ( ! ref( $msg->[ MSG_DATA ]->[0] ) ) {
+			push( @edges, [ $msg->[ MSG_DATA ]->[0], $msg->[ MSG_DATA ]->[1] ] );
+		} else {
+			@edges = @{ $msg->[ MSG_DATA ] };
+		}
+
+		# Do it!
+		foreach my $e ( @edges ) {
+			# Delete this edge from our graph
+			$_[HEAP]->{'GRAPH'}->delete_edge( $e->[0], $e->[1] );
+
+			# Update the APSP
+			$_[HEAP]->{'APSP'} = $_[HEAP]->{'GRAPH'}->APSP_Floyd_Warshall;
+
+			# Are any kernels isolated now ( no way to reach them )
+			foreach my $kernel ( $_[HEAP]->{'GRAPH'}->vertices ) {
+				if ( $kernel eq $_[HEAP]->{'MYKERNEL'} ) {
+					next;
+				}
+
+				if ( ! $_[HEAP]->{'APSP'}->is_reachable( $_[HEAP]->{'MYKERNEL'}, $kernel ) ) {
+					# This kernel is isolated, delete it
+					$_[HEAP]->{'GRAPH'}->delete_vertex( $kernel );
+
+					# Update the APSP
+					$_[HEAP]->{'APSP'} = $_[HEAP]->{'GRAPH'}->APSP_Floyd_Warshall;
+				}
+			}
+		}
+	}
+
+	# Do whatever is necessary with this packet
+	SendMessage( $_[HEAP], $msg );
+
+	# All done!
+	return 1;
+}
+
+# Processes POST messages
+sub A_POST {
+	# ARG0 = data packet, ARG1 = wheel id
+	my $msg = $_[ARG0];
+
+	# Is it for us?
+	if ( FindOurself( $_[HEAP]->{'MYKERNEL'}, $msg->[ MSG_TO ] ) ) {
+		# Send it off!
+		post_self(
+			$_[HEAP],
+			POE::Component::Lightspeed::Hack::Session->new( @{ $msg->[ MSG_DATA ]->[ POST_FROM ] } ),
+			$msg->[ MSG_DATA ]->[ POST_TO ],
+			$msg->[ MSG_DATA ]->[ POST_ARGS ],
+		);
+	}
+
+	# Do whatever is necessary with this packet
+	SendMessage( $_[HEAP], $msg );
+
+	# All done!
+	return 1;
+}
+
+# Processes CALL messages
+sub A_CALL {
+	# ARG0 = data packet, ARG1 = wheel id
+	my $msg = $_[ARG0];
+#warn "CALL - packet:", Data::Dumper::Dumper( $msg );
+	# Is it for us?
+	if ( FindOurself( $_[HEAP]->{'MYKERNEL'}, $msg->[ MSG_TO ] ) ) {
+		# Create the fake session
+		my $fake_session = POE::Component::Lightspeed::Hack::Session->new( @{ $msg->[ MSG_DATA ]->[ CALL_FROM ] } );
+
+		# Which session are we sending this to?
+		my @sessions = ();
+		if ( ! ref( $msg->[ MSG_DATA ]->[ CALL_TO ]->[ DEST_SESSION ] ) ) {
+			# Are we doing all sessions?
+			if ( $msg->[ MSG_DATA ]->[ CALL_TO ]->[ DEST_SESSION ] eq '*' ) {
+				@sessions = GetSessions();
+			} else {
+				$sessions[0] = $msg->[ MSG_DATA ]->[ CALL_TO ]->[ DEST_SESSION ];
+			}
+		} else {
+			@sessions = @{ $msg->[ MSG_DATA ]->[ CALL_TO ]->[ DEST_SESSION ] };
+		}
+
+		# Do our job!
+		foreach my $ses ( @sessions ) {
+			# Check if it's a multiple-alias session
+			if ( ref( $ses ) ) {
+				$ses = $ses->[0];
+			}
+
+			# Okay, check if the session exists
+			my $sesreal = $_[KERNEL]->_resolve_session( $ses );
+			if ( ! defined $sesreal ) {
+				# Error...
+				if ( DEBUG ) {
+					warn "Unknown session: $ses\n" . Data::Dumper::Dumper( $msg );
+				}
+				next;
+			}
+
+			# Call it!
+			my $args = $_[KERNEL]->lightspeed_fake_call(
+				$fake_session,
+				$sesreal,
+				$msg->[ MSG_DATA ]->[ CALL_TO ]->[ DEST_STATE ],
+				$msg->[ MSG_DATA ]->[ CALL_ARGS ],
+			);
+
+			# Return it back to the RSVP
+			SendMessage( $_[HEAP], [
+				$msg->[ MSG_DATA ]->[ CALL_RSVP ]->[ DEST_KERNEL ],		# MSG_TO
+				undef,								# MSG_FROM
+				ACTION_CALLREPLY,						# MSG_ACTION
+				[								# MSG_DATA
+					$msg->[ MSG_DATA ]->[ CALL_RSVP ],			# CALLREPLY_TO
+					[							# CALLREPLY_FROM
+						$_[HEAP]->{'MYKERNEL'},				# FROM_KERNEL
+						$ses,						# FROM_SESSION
+						$msg->[ MSG_DATA ]->[ CALL_TO ]->[ DEST_STATE ],# FROM_STATE
+						'Lightspeed.pm',				# FROM_FILE
+						1,						# FROM_LINE
+					],
+					$args,							# CALLREPLY_ARGS
+				],
+			] );
+		}
+	}
+
+	# Do whatever is necessary with this packet
+	SendMessage( $_[HEAP], $msg );
+
+	# All done!
+	return 1;
+}
+
+# processes INTROSPECTION messages
+sub A_INTROSPECTION {
+	# ARG0 = data packet, ARG1 = wheel id
+	my $msg = $_[ARG0];
+
+	# Is it for us?
+	if ( FindOurself( $_[HEAP]->{'MYKERNEL'}, $msg->[ MSG_TO ] ) ) {
+		# Okay, is it a SESSION or STATE request?
+		if ( $msg->[ MSG_DATA ]->[ INTROSPECTION_WHAT ] eq 'SESSION' ) {
+			# Get the list of sessions and send it back!
+			my @sessions = GetSessions();
+
+			# Now, is the rsvp towards ourself?
+			if ( FindOurself( $_[HEAP]->{'MYKERNEL'}, $msg->[ MSG_DATA ]->[ INTROSPECTION_RSVP ]->[ DEST_KERNEL ] ) ) {
+				# Create the fake session
+				my $fake_session = POE::Component::Lightspeed::Hack::Session->new( @{ $msg->[ MSG_DATA ]->[ INTROSPECTION_FROM ] } );
+
+				# Send it to ourself...
+				post_self(
+					$_[HEAP],
+					$fake_session,
+					$msg->[ MSG_DATA ]->[ INTROSPECTION_RSVP ],
+					[ $_[HEAP]->{'MYKERNEL'}, \@sessions ],
+				);
+			}
+
+			# Send it back to the RSVP!
+			SendMessage( $_[HEAP], [
+				$msg->[ MSG_DATA ]->[ INTROSPECTION_RSVP ]->[ DEST_KERNEL ],	# MSG_TO
+				undef,								# MSG_FROM
+				ACTION_POST,							# MSG_ACTION
+				[								# MSG_DATA
+					$msg->[ MSG_DATA ]->[ INTROSPECTION_RSVP ],		# POST_TO
+					$msg->[ MSG_DATA ]->[ INTROSPECTION_FROM ],		# POST_FROM
+					[ $_[HEAP]->{'MYKERNEL'}, \@sessions ],			# POST_ARGS
+				],
+			] );
+		} elsif ( $msg->[ MSG_DATA ]->[ INTROSPECTION_WHAT ] eq 'STATE' ) {
+			# Loop through the sessions
+			my %data = ();
+			my @sessions = ();
+
+			# Is it the special '*' session flag?
+			if ( ! ref( $msg->[ MSG_DATA ]->[ INTROSPECTION_ARGS ] ) ) {
+				if ( $msg->[ MSG_DATA ]->[ INTROSPECTION_ARGS ] eq '*' ) {
+					@sessions = GetSessions();
+				} else {
+					$sessions[0] = $msg->[ MSG_DATA ]->[ INTROSPECTION_ARGS ];
+				}
+			} else {
+				@sessions = @{ $msg->[ MSG_DATA ]->[ INTROSPECTION_ARGS ] };
+			}
+
+			foreach my $ses ( @sessions ) {
+				# Check if it's a multiple-alias session
+				if ( ref( $ses ) ) {
+					$ses = $ses->[0];
+				}
+
+				# Is it a valid session?
+				my $sesreal = $_[KERNEL]->_resolve_session( $ses );
+				if ( ! defined $sesreal ) {
+					# Error...
+					if ( DEBUG ) {
+						warn "Unknown session: $ses";
+					}
+					next;
+				}
+
+				# Okay, we have the session reference, HAX0R TIME!
+				$data{ $ses } = [ keys %{ $sesreal->[ POE::Session::SE_STATES ] } ];
+			}
+
+			# Now, is the rsvp towards ourself?
+			if ( FindOurself( $_[HEAP]->{'MYKERNEL'}, $msg->[ MSG_DATA ]->[ INTROSPECTION_RSVP ]->[ DEST_KERNEL ] ) ) {
+				# Create the fake session
+				my $fake_session = POE::Component::Lightspeed::Hack::Session->new( @{ $msg->[ MSG_DATA ]->[ INTROSPECTION_FROM ] } );
+
+				# Send it to ourself...
+				post_self(
+					$_[HEAP],
+					$fake_session,
+					$msg->[ MSG_DATA ]->[ INTROSPECTION_RSVP ],
+					[ $_[HEAP]->{'MYKERNEL'}, \%data ],
+				);
+			}
+
+			# Send it back to the RSVP!
+			SendMessage( $_[HEAP], [
+				$msg->[ MSG_DATA ]->[ INTROSPECTION_RSVP ]->[ DEST_KERNEL ],	# MSG_TO
+				undef,								# MSG_FROM
+				ACTION_POST,							# MSG_ACTION
+				[								# MSG_DATA
+					$msg->[ MSG_DATA ]->[ INTROSPECTION_RSVP ],		# POST_TO
+					$msg->[ MSG_DATA ]->[ INTROSPECTION_FROM ],		# POST_FROM
+					[ $_[HEAP]->{'MYKERNEL'}, \%data ],			# POST_ARGS
+				],
+			] );
+		} else {
+			# Unknown request!
+			if ( DEBUG ) {
+				warn "Unknown INTROSPECTION_WHAT -> " . Data::Dumper::Dumper( $msg );
+			}
+		}
+	}
+
+	# Do whatever is necessary with this packet
+	SendMessage( $_[HEAP], $msg );
+}
+
+# Helper sub to get the list of sessions with aliases
+sub GetSessions {
+	# Big thanks to POE::API::Peek for this code
+
+	my @sessions = ();
+	my $kr_sessions = $poe_kernel->[ POE::Kernel::KR_SESSIONS ];
+	foreach my $key ( keys %$kr_sessions ) {
+		# Skip over the Kernel itself
+		next if $key =~ /POE::Kernel/;
+
+		# Get the alias
+		my @list = $poe_kernel->alias_list( $kr_sessions->{ $key }->[ POE::Kernel::SS_SESSION ] );
+		if ( defined $list[0] ) {
+			push( @sessions, [ @list ] );
+		}
+	}
+
+	# All done!
+	return @sessions;
+}
+
+# Helper to find if we are the target of a destination
+sub FindOurself {
+	my( $our, $dest ) = @_;
+
+	# If destination is unspecified, then obviously not us
+	if ( ! defined $dest ) {
+		if ( DEBUG ) {
+			Carp::confess( "Undefined destination!" );
+		}
+		return 0;
+	}
+
+	# Is it an arrayref?
+	if ( ref( $dest ) ) {
+		# Find ourself in it...
+		foreach my $t ( @$dest ) {
+			if ( $our eq $t ) {
+				return 1;
+			}
+		}
+
+		# Got here, didn't find it
+		return 0;
+	} else {
+		# Is it *
+		if ( $dest eq '*' ) {
+			return 1;
+		} elsif ( $dest eq $our ) {
+			return 1;
+		} else {
+			return 0;
+		}
+	}
+}
+
 # Helper subroutine to parse destination scalars
 sub ValidateDestination {
 	my $dest = shift;
@@ -535,99 +1050,6 @@ sub ValidateDestination {
 	return $dest;
 }
 
-# A link is up!
-sub Link_Up {
-	# ARG0 = wheel id, ARG1 = our kernel name, ARG2 = remote name, ARG3 = Server/Client
-	my( $ses, $id, $kernel, $remotekernel, $type ) = @_[ SENDER, ARG0 .. ARG3 ];
-	$ses = $ses->ID;
-
-	# The kernel name cannot include * or / or ,
-	if ( $kernel =~ tr|*/,|| ) {
-		die "The kernel name cannot include '*' or '/' or ','";
-	}
-
-	# Ah, now we know our name :)
-	if ( ! defined $_[HEAP]->{'MYKERNEL'} ) {
-		$_[HEAP]->{'MYKERNEL'} = $kernel;
-	} else {
-		# Sanity check...
-		if ( $_[HEAP]->{'MYKERNEL'} ne $kernel ) {
-			die 'Using different kernel names in the same process is not supported!';
-		}
-	}
-
-	# We do some sanity checks here
-	if ( $_[HEAP]->{'GRAPH'}->has_vertex( $remotekernel ) ) {
-		# A new node wants to join, but we already have the same kernel name!
-		warn "Detected same kernel name in use - $remotekernel";
-		$_[KERNEL]->call( $ses, 'killclient', $id );
-		return undef;
-	}
-
-	# Store it!
-	push( @{ $_[HEAP]->{'SID_WID'}->{ $ses } }, $id );
-	$_[HEAP]->{'WID_SID'}->{ $id } = $ses;
-	$_[HEAP]->{'WID_TYPE'}->{ $id } = $type;
-	$_[HEAP]->{'KERNEL_WID'}->{ $remotekernel } = $id;
-	$_[HEAP]->{'WID_KERNEL'}->{ $id } = $remotekernel;
-
-	# Get the edges
-	my @edges = $_[HEAP]->{'GRAPH'}->edges();
-	if ( scalar( @edges ) == 1 ) {
-		@edges = ( $edges[0]->[0], $edges[0]->[1] );
-	}
-
-	# Send our kernel info
-	$_[KERNEL]->post( $ses, 'send', $id, [
-		$remotekernel,	# MSG_TO
-		$kernel,	# MSG_FROM
-		ACTION_HELLO,	# MSG_ACTION
-		\@edges,	# MSG_DATA
-	] );
-}
-
-# A link is down!
-sub Link_Down {
-	# ARG0 = wheel id
-	my( $ses, $id ) = @_[ SENDER, ARG0 ];
-	$ses = $ses->ID;
-	my $kernel = $_[HEAP]->{'WID_KERNEL'}->{ $id };
-
-	# Remove it from that session
-	$_[HEAP]->{'SID_WID'}->{ $ses } = [ grep { $_ ne $id } @{ $_[HEAP]->{'SID_WID'}->{ $ses } } ];
-	delete $_[HEAP]->{'WID_SID'}->{ $id };
-	delete $_[HEAP]->{'SID_WID'}->{ $ses } if scalar( @{ $_[HEAP]->{'SID_WID'}->{ $ses } } ) == 0;
-
-	# Let ACTION_ROUTEDEL do some work
-	$_[KERNEL]->call( $_[SESSION], 'ACTION_' . ACTION_ROUTEDEL, [
-		$_[HEAP]->{'MYKERNEL'},			# MSG_TO
-		undef,					# MSG_FROM
-		undef,					# MSG_ACTION
-		[ $_[HEAP]->{'MYKERNEL'}, $kernel ],	# MSG_DATA
-	] );
-
-	# Remove it from the wheel mapping
-	delete $_[HEAP]->{'KERNEL_WID'}->{ $kernel };
-	delete $_[HEAP]->{'WID_KERNEL'}->{ $id };
-
-	# Are we responsible?
-	if ( $_[HEAP]->{'WID_TYPE'}->{ $id } eq 'Server' or ! $_[HEAP]->{'GRAPH'}->has_vertex( $kernel ) ) {
-		# Let all kernels know
-		SendMessage( $_[HEAP], [
-			undef,					# MSG_TO
-			undef,					# MSG_FROM
-			ACTION_ROUTEDEL,			# MSG_ACTION
-			[ $kernel, $_[HEAP]->{'MYKERNEL'} ],	# MSG_DATA
-		] );
-	}
-
-	# One last piece of deletion
-	delete $_[HEAP]->{'WID_TYPE'}->{ $id };
-
-	# All done!
-	return 1;
-}
-
 # Sends the message!
 sub SendMessage {
 	my( $heap, $msg ) = @_;
@@ -636,13 +1058,13 @@ sub SendMessage {
 	if ( ! defined $msg->[ MSG_TO ] ) {
 		$msg->[ MSG_TO ] = [ $heap->{'GRAPH'}->vertices ];
 	} elsif ( ! ref( $msg->[ MSG_TO ] ) ) {
-		# Convert it into an array
-		$msg->[ MSG_TO ] = [ $msg->[ MSG_TO ] ];
-	}
-
-	# Add the From
-	if ( ! defined $msg->[ MSG_FROM ] ) {
-		$msg->[ MSG_FROM ] = $heap->{'MYKERNEL'};
+		# Is it the broadcast kernel?
+		if ( $msg->[ MSG_TO ] eq '*' ) {
+			$msg->[ MSG_TO ] = [ $heap->{'GRAPH'}->vertices ];
+		} else {
+			# Convert it into an array
+			$msg->[ MSG_TO ] = [ $msg->[ MSG_TO ] ];
+		}
 	}
 
 	# For starters, strip out our kernel if it is there + remove any invalid kernels
@@ -653,6 +1075,11 @@ sub SendMessage {
 		return 1;
 	}
 
+	# Add the From
+	if ( ! defined $msg->[ MSG_FROM ] ) {
+		$msg->[ MSG_FROM ] = $heap->{'MYKERNEL'};
+	}
+
 	# Add the realto field
 	if ( ! defined $msg->[ MSG_REALTO ] ) {
 		$msg->[ MSG_REALTO ] = [ @{ $msg->[ MSG_TO ] } ];
@@ -724,305 +1151,6 @@ sub SendMessage {
 	return 1;
 }
 
-# Processes a HELLO message
-sub HELLO {
-	# ARG0 = data packet, ARG1 = wheel id
-	my $msg = $_[ARG0];
-
-	# Add it to our graph
-	$_[HEAP]->{'GRAPH'}->add_weighted_edge( $_[HEAP]->{'MYKERNEL'}, $msg->[ MSG_FROM ], 1 );
-
-	# Update the APSP
-	$_[HEAP]->{'APSP'} = $_[HEAP]->{'GRAPH'}->APSP_Floyd_Warshall;
-
-	# Store the new edges
-	my @edges = ();
-	if ( scalar( @{ $msg->[ MSG_DATA ] } ) > 0 ) {
-		if ( ! ref( $msg->[ MSG_DATA ]->[0] ) ) {
-			# Single edge
-			if ( ! $_[HEAP]->{'GRAPH'}->has_edge( $msg->[ MSG_DATA ]->[0], $msg->[ MSG_DATA ]->[1] ) ) {
-				push( @edges, [ $msg->[ MSG_DATA ]->[0], $msg->[ MSG_DATA ]->[1] ] );
-			}
-		} else {
-			foreach my $e ( @{ $msg->[ MSG_DATA ] } ) {
-				# Do we know about this already?
-				if ( ! $_[HEAP]->{'GRAPH'}->has_edge( $e->[0], $e->[1] ) ) {
-					# This is new to us
-					push( @edges, $e );
-				}
-			}
-		}
-	}
-
-	# This sucks.
-	my @targets = $_[HEAP]->{'GRAPH'}->vertices;
-	@targets = grep { $_ ne $msg->[ MSG_FROM ] } @targets;
-
-	# Broadcast this new info to our network
-	if ( scalar( @edges ) > 0 ) {
-		# Add them all to the graph
-		foreach my $e ( @edges ) {
-			$_[HEAP]->{'GRAPH'}->add_weighted_edge( $e->[0], $e->[1], 1 );
-		}
-
-		# Update the APSP
-		$_[HEAP]->{'APSP'} = $_[HEAP]->{'GRAPH'}->APSP_Floyd_Warshall;
-
-		# Add our link to the edges
-		push( @edges, [ $_[HEAP]->{'MYKERNEL'}, $msg->[ MSG_FROM ] ] );
-	} else {
-		# Add our link to the edges
-		@edges = ( $_[HEAP]->{'MYKERNEL'}, $msg->[ MSG_FROM ] );
-	}
-
-	# Inform the network about our new link!
-	SendMessage( $_[HEAP], [
-		\@targets,		# MSG_TO
-		undef,			# MSG_FROM
-		ACTION_ROUTENEW,	# MSG_ACTION
-		\@edges,		# MSG_DATA
-	] );
-
-	# All done!
-	return 1;
-}
-
-# Processes ROUTE_NEW messages
-sub ROUTE_NEW {
-	# ARG0 = data packet, ARG1 = wheel id
-	my $msg = $_[ARG0];
-
-	# Is it for us?
-	if ( FindOurself( $_[HEAP]->{'MYKERNEL'}, $msg->[ MSG_TO ] ) ) {
-		# Process each edge
-		my $edges = [];
-		if ( ! ref( $msg->[ MSG_DATA ]->[0] ) ) {
-			push( @$edges, [ $msg->[ MSG_DATA ]->[0], $msg->[ MSG_DATA ]->[1] ] );
-		} else {
-			$edges = $msg->[ MSG_DATA ];
-		}
-
-		# Do it!
-		foreach my $e ( @$edges ) {
-			# Do we know about this already?
-			if ( ! $_[HEAP]->{'GRAPH'}->has_edge( $e->[0], $e->[1] ) ) {
-				# Add it!
-				$_[HEAP]->{'GRAPH'}->add_weighted_edge( $e->[0], $e->[1], 1 );
-			} else {
-				if ( DEBUG ) {
-					warn "ROUTE_NEW -> discarding known edge: $e->[0] <=> $e->[1]";
-				}
-			}
-		}
-
-		# Update the APSP
-		$_[HEAP]->{'APSP'} = $_[HEAP]->{'GRAPH'}->APSP_Floyd_Warshall;
-	}
-
-	# Do whatever is necessary with this packet
-	SendMessage( $_[HEAP], $msg );
-
-	# All done!
-	return 1;
-}
-
-# Processes a ROUTE_DEL message
-sub ROUTE_DEL {
-	# ARG0 = data packet, ARG1 = wheel id
-	my $msg = $_[ARG0];
-
-	# Is it for us?
-	if ( FindOurself( $_[HEAP]->{'MYKERNEL'}, $msg->[ MSG_TO ] ) ) {
-		# Process each edge
-		my @edges = ();
-		if ( ! ref( $msg->[ MSG_DATA ]->[0] ) ) {
-			push( @edges, [ $msg->[ MSG_DATA ]->[0], $msg->[ MSG_DATA ]->[1] ] );
-		} else {
-			@edges = @{ $msg->[ MSG_DATA ] };
-		}
-
-		# Do it!
-		foreach my $e ( @edges ) {
-			# Delete this edge from our graph
-			$_[HEAP]->{'GRAPH'}->delete_edge( $e->[0], $e->[1] );
-
-			# Update the APSP
-			$_[HEAP]->{'APSP'} = $_[HEAP]->{'GRAPH'}->APSP_Floyd_Warshall;
-
-			# Are any kernels isolated now ( no way to reach them )
-			foreach my $kernel ( $_[HEAP]->{'GRAPH'}->vertices ) {
-				if ( $kernel eq $_[HEAP]->{'MYKERNEL'} ) {
-					next;
-				}
-
-				if ( ! $_[HEAP]->{'APSP'}->is_reachable( $_[HEAP]->{'MYKERNEL'}, $kernel ) ) {
-					# This kernel is isolated, delete it
-					$_[HEAP]->{'GRAPH'}->delete_vertex( $kernel );
-
-					# Update the APSP
-					$_[HEAP]->{'APSP'} = $_[HEAP]->{'GRAPH'}->APSP_Floyd_Warshall;
-				}
-			}
-		}
-	}
-
-	# Do whatever is necessary with this packet
-	SendMessage( $_[HEAP], $msg );
-
-	# All done!
-	return 1;
-}
-
-
-# Helper to find if we are the target of a destination
-sub FindOurself {
-	my( $our, $dest ) = @_;
-
-	# If destination is unspecified, then obviously not us
-	if ( ! defined $dest ) {
-		if ( DEBUG ) {
-			Carp::confess( "Undefined destination!" );
-		}
-		return 0;
-	}
-
-	# Is it an arrayref?
-	if ( ref( $dest ) ) {
-		# Find ourself in it...
-		foreach my $t ( @$dest ) {
-			if ( $our eq $t ) {
-				return 1;
-			}
-		}
-
-		# Got here, didn't find it
-		return 0;
-	} else {
-		# Is it *
-		if ( $dest eq '*' ) {
-			return 1;
-		} elsif ( $dest eq $our ) {
-			return 1;
-		} else {
-			return 0;
-		}
-	}
-}
-
-# Processes POST messages
-sub POST {
-	# ARG0 = data packet, ARG1 = wheel id
-	my $msg = $_[ARG0];
-
-	# Is it for us?
-	if ( FindOurself( $_[HEAP]->{'MYKERNEL'}, $msg->[ MSG_TO ] ) ) {
-		# Send it off!
-		post_self(
-			$_[HEAP],
-			POE::Component::Lightspeed::Hack::Session->new( @{ $msg->[ MSG_DATA ]->[ POST_FROM ] } ),
-			$msg->[ MSG_DATA ]->[ POST_TO ],
-			$msg->[ MSG_DATA ]->[ POST_ARGS ],
-		);
-	}
-
-	# Do whatever is necessary with this packet
-	SendMessage( $_[HEAP], $msg );
-
-	# All done!
-	return 1;
-}
-
-# Processes CALL messages
-sub CALL {
-	# ARG0 = data packet, ARG1 = wheel id
-	my $msg = $_[ARG0];
-warn "CALL - packet:", Data::Dumper::Dumper( $msg );
-	# Is it for us?
-	if ( FindOurself( $_[HEAP]->{'MYKERNEL'}, $msg->[ MSG_TO ] ) ) {
-		# Create the fake session
-		my $fake_session = POE::Component::Lightspeed::Hack::Session->new( @{ $msg->[ MSG_DATA ]->[ CALL_FROM ] } );
-
-		# Which session are we sending this to?
-		our @sessions = ();
-		if ( ! ref( $msg->[ MSG_DATA ]->[ CALL_TO ]->[ DEST_SESSION ] ) ) {
-			# Are we doing all sessions?
-			if ( $msg->[ MSG_DATA ]->[ CALL_TO ]->[ DEST_SESSION ] eq '*' ) {
-				@sessions = GetSessions();
-			} else {
-				$sessions[0] = $msg->[ MSG_DATA ]->[ CALL_TO ]->[ DEST_SESSION ];
-			}
-		} else {
-			@sessions = @{ $msg->[ MSG_DATA ]->[ CALL_TO ]->[ DEST_SESSION ] };
-		}
-
-		# Do our job!
-		foreach my $ses ( @sessions ) {
-			# Okay, check if the session exists
-			my $sesreal = $_[KERNEL]->_resolve_session( $ses );
-			if ( ! defined $sesreal ) {
-				# Error...
-				if ( DEBUG ) {
-					warn "Unknown session: $ses\n" . Data::Dumper::Dumper( $msg );
-				}
-				next;
-			}
-
-			# Call it!
-			my $args = $_[KERNEL]->lightspeed_fake_call(
-				$fake_session,
-				$sesreal,
-				$msg->[ MSG_DATA ]->[ CALL_TO ]->[ DEST_STATE ],
-				$msg->[ MSG_DATA ]->[ CALL_ARGS ],
-			);
-
-			# Return it back to the RSVP
-			SendMessage( $_[HEAP], [
-				$msg->[ MSG_DATA ]->[ CALL_RSVP ]->[ DEST_KERNEL ],		# MSG_TO
-				undef,								# MSG_FROM
-				ACTION_CALLREPLY,						# MSG_ACTION
-				[								# MSG_DATA
-					$msg->[ MSG_DATA ]->[ CALL_RSVP ],			# CALLREPLY_TO
-					[							# CALLREPLY_FROM
-						$_[HEAP]->{'MYKERNEL'},				# FROM_KERNEL
-						$ses,						# FROM_SESSION
-						$msg->[ MSG_DATA ]->[ CALL_TO ]->[ DEST_STATE ],# FROM_STATE
-						'Lightspeed.pm',				# FROM_FILE
-						1,						# FROM_LINE
-					],
-					$args,							# CALLREPLY_ARGS
-				],
-			] );
-		}
-	}
-
-	# Do whatever is necessary with this packet
-	SendMessage( $_[HEAP], $msg );
-
-	# All done!
-	return 1;
-}
-
-# Helper sub to get the list of sessions with aliases
-sub GetSessions {
-	# Big thanks to POE::API::Peek for this code
-
-	my @sessions = ();
-	my $kr_sessions = $poe_kernel->[ POE::Kernel::KR_SESSIONS ];
-	foreach my $key ( %$kr_sessions ) {
-		# Skip over the Kernel itself
-		next if $key =~ /POE::Kernel/;
-
-		# Get the alias
-		my @list = $poe_kernel->alias_list( $kr_sessions->{ $key }->[0] );
-		if ( defined $list[0] ) {
-			push( @sessions, $list[0] );
-		}
-	}
-
-	# All done!
-	return @sessions;
-}
-
 # End of module
 1;
 __END__
